import pytest
from preprocess.cleaner_imp import CleanerImp


class TestCleaner:

    def setup_method(self):
        self.cleaner = CleanerImp()

    def test_init_cleaner(self):
        assert self.cleaner is not None

    def test_can_create_bigram(self):
        sentence = "I like to eat ice cream in new york."
        expected = "I like to eat ice_cream in new_york."

        result = self.cleaner.bigrams(sentence)

        assert result == expected


    def test_can_create_trigram(self):
        sentence = "Cloud computing is benefiting major manufacturing companies"
        expected = "Cloud_computing is benefiting major_manufacturing_companies"

        result = self.cleaner.bigrams(sentence)

        assert result == expected

    def test_cleaner_makes_words_to_lowercase(self):
        text = "Hi My Name Is Torben"

        lower_case_text = self.cleaner.to_lower(text)

        expected_text = "hi my name is torben"
        assert expected_text == lower_case_text

    def test_cleaner_removes_single_special_character(self):
        text = "I have five $ in my wallet"

        cleaned_text = self.cleaner.remove_special_characters(text)

        expexted_text = "I have five  in my wallet"
        assert expexted_text == cleaned_text

    def test_cleaner_removes_many_special_characters(self):
        text = 'My "#¤&%/)(name! \'is@ torben?¨^`'
        expexted_text = "My name is torben"

        cleaned_text = self.cleaner.remove_special_characters(text)

        assert expexted_text == cleaned_text

    def test_cleaner_removes_many_special_characters_keep_punct(self):
        text = 'My "#¤&%/)(name! \'is@ torben.?¨^`'
        expexted_text = "My name is torben."

        cleaned_text = self.cleaner.remove_special_characters(text)

        assert expexted_text == cleaned_text

    def test_cleaner_should_textify_single_digit(self):
        text = "I have 5 dollars in my pocket"
        expected_text = "I have five dollars in my pocket"

        cleaned_text = self.cleaner.numbers_to_text(text)

        assert expected_text == cleaned_text

    def test_cleaner_should_textity_long_number(self):
        text = "34567"
        expected_text = "three four five six seven"

        cleaned_text = self.cleaner.numbers_to_text(text)

        assert expected_text == cleaned_text

    def test_cleaner_should_textity_double_digits(self):
        text = "54"
        expected_text = "five four"

        cleaned_text = self.cleaner.numbers_to_text(text)

        assert expected_text == cleaned_text

    def test_remove_duplicates_in_string_list(self):
        text_list = ['Hej mit navn er Torben.',
                     'Hej mit navn er Solvej.',
                     'Hej mit navn er Torben.']
        expected_text = ['Hej mit navn er Torben.',
                         'Hej mit navn er Solvej.']

        duplicates_removed = self.cleaner.remove_duplicates(text_list)

        assert set(expected_text) == set(duplicates_removed)

    def test_lemmatize_walking_to_walk(self):
        sentence = "walking"

        lemma_word = self.cleaner.lemmatize(sentence)

        assert lemma_word == 'walk'

    def test_lemmatize_feet_to_foot(self):
        sentence = "feet"

        lemma_word = self.cleaner.lemmatize(sentence)

        assert lemma_word == 'foot'

    def test_lemmatize_multiple_words(self):
        sentence = "walking feet"

        lemmatized = self.cleaner.lemmatize(sentence)

        assert lemmatized == "walk foot"

    def xtest_valuta_gets_lemmatised_to_text(self):
        sentence = "Many $"

        lemmatized = self.cleaner.lemmatize(sentence)

        assert lemmatized == "many dollar"

# Test ideas



# !/usr/bin/python3
import os
import sys
from argparse import ArgumentParser

import spacy
from spacy.cli import download

from loader.file_loader import load_json
from mi_graph import api, pipeline, __version__
from resources.error import error

MODE_FILE = 'file'
MODE_SERVE = 'flask'


def setup_parser(parser: ArgumentParser) -> ArgumentParser:
    parser.prog = "mi-graph"

    parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + __version__)
    parser.add_argument("mode", choices=[MODE_FILE, MODE_SERVE],
                        help='Choose if you want to process a file, or run the program as a rest api')
    parser.add_argument("--file", "-f", help="Please indicate the json file you want to process.",
                        metavar='')  # makes help prettier

    parser.add_argument("--visualisation", action="store_true", default=False,
                        help="This option visualizes the graph with plotly, "
                             "after the script has run")
    return parser


def make_path(path):
    """ Returns a uniformly real, absolute filesystem path."""

    # ~/directory -> /home/user/directory
    path = os.path.expanduser(path)
    # A/.//B -> A/B
    path = os.path.normpath(path)
    # Resolve symbolic links
    path = os.path.realpath(path)
    # Ensure path is absolute
    path = os.path.abspath(path)

    return path


def ensure_models_installed():
    okcyan = '\033[96m'
    endc = '\033[0m'

    if not spacy.util.is_package("en_core_web_sm"):
        print(f"{okcyan}Missing models. Install spacy en-core-web-sm model{endc}")
        download("en_core_web_sm")


def cli():
    # Setup the parser and parse the arguments
    parser = setup_parser(ArgumentParser())

    args = parser.parse_args()

    ensure_models_installed()

    if args.mode == MODE_SERVE:
        api.start_api()
        sys.exit(1)

    # we know it is a file input
    if not args.file:
        error("Specify a file, you want to parse pls")
        parser.print_help()
        sys.exit(0)

    # Make path of the input file real and absolute. Ensures cross platform compatibility
    path_to_file = make_path(args.file)

    print(path_to_file)
    input_file = open(path_to_file, encoding='utf-16')
    manual = load_json(input_file)

    pipeline.run(manual, visualize=args.visualisation)



import json
from resources.json_wrapper import Content, Manual


def load_json(io_stream):
    if io_stream:
        with io_stream as json_file:
            data = json.load(json_file)
        return instantiate_content_object(data)
    raise IOError


def instantiate_content_object(data: dict):
    con = Content(data["content"])
    return Manual(publisher=con.published_by,
                  published_at=con.published_at,
                  title=con.title,
                  sections=con.sections)



from word_embedding.grammarcategories import GrammarCategories
from word_embedding.dependency import Dependency


class Token:
    """
    Class token is implementation of token for the word embeddings
    """
    pos_tag: GrammarCategories
    dep: Dependency
    name = ""

    def __init__(self, name, pos_tag=None, dep=None):
        self.name = name
        self.pos_tag = pos_tag
        self.dep = dep

    def __eq__(self, other):
        return self.name == other.name

    def __repr__(self):
        return f"Token: name:{self.name}"



from enum import Enum, auto


class Dependency(Enum):
    """
    Dependencies for the word embeddings used in general
    """
    root = auto()
    nsubj = auto()
    acl = auto()
    acomp = auto()
    advcl = auto()
    advmod = auto()
    agent = auto()
    amod = auto()
    appos = auto()
    attr = auto()
    aux = auto()
    auxpass = auto()
    case = auto()
    cc = auto()
    ccomp = auto()
    compound = auto()
    conj = auto()
    cop = auto()
    csubj = auto()
    csubjpass = auto()
    dative = auto()
    dep = auto()
    det = auto()
    dobj = auto()
    expl = auto()
    intj = auto()
    mark = auto()
    meta = auto()
    neg = auto()
    nn = auto()
    nounmod = auto()
    npmod = auto()
    nsubjpass = auto()
    nummod = auto()
    oprd = auto()
    obj = auto()
    obl = auto()
    parataxis = auto()
    pcomp = auto()
    pobj = auto()
    poss = auto()
    preconj = auto()
    prep = auto()
    prt = auto()
    punct = auto()
    quantmod = auto()
    relcl = auto()
    xcomp = auto()
    other = auto()



from word_embedding.dependency import Dependency


class Converter():
    """
    Converter class for the dependencies in spacy to known knox dependencies
    """
    @staticmethod
    def dependency(dep: str) -> Dependency:
        converter = {
            "nsubj": Dependency.nsubj,
            "pobj": Dependency.pobj,
            "aux": Dependency.aux,
            "ROOT": Dependency.root,
            "prep": Dependency.prep,
            "pcomp": Dependency.pcomp,
            "compound": Dependency.compound,
            "dobj": Dependency.dobj,
            "quantmod": Dependency.quantmod,
        }

        if dep not in converter.keys():
            return Dependency.other
        return converter[dep]



import json
import os.path
from io import StringIO

from loader.file_loader import load_json
from resources import knox_triples
from resources.json_wrapper import Manual
from word_embedding.token import Token
from resources.knowledgegraph_info_container import KnowledgeGraphInfo
from word_embedding.dependency import Dependency

from mi_graph.knowledge_graph import KnowledgeGraph


class TestKnowledgeGraph:

    def setup_method(self):
        self.kg = KnowledgeGraph()

    def teardown_method(self):
        self.kg.knowledge_graph_triples = []
        # Erase file after tests
        if os.path.isfile("test_files/testfile.csv"):
            os.remove("test_files/testfile.csv")

    # Reafactor to send to database, instead of saving to csv file
    def xtest_creates_csv_file_in_correct_folder(self):
        # Arrange
        sentences = []
        manual = Manual()
        kg_info = KnowledgeGraphInfo(sentences, manual)

        # Act
        self.kg.generate_triples(kg_info)
        self.kg.save_to_csv("test_files/testfile.csv")
        file_exists = os.path.isfile("test_files/testfile.csv")

        # Assert
        assert file_exists

    def test_validate_analyse_single_sentence(self):
        # Arrange
        sentence = [[Token('Pump', dep=Dependency.nsubj),
                     Token('develop', dep=Dependency.root),
                     Token('well', dep=Dependency.obj)]]

        manual = Manual()

        kg_info = KnowledgeGraphInfo(sentence, manual)

        # Act
        self.kg.generate_triples(kg_info)
        result = self.kg.knowledge_graph_triples

        # Assert
        assert isinstance(result[0], knox_triples.SentenceTriple)
        assert len(result) == 1

    def test_validate_analyse_multiple_sentences(self):
        # Arrange
        sentences = [[Token('Martin', dep=Dependency.nsubj),
                      Token('likes', dep=Dependency.root),
                      Token('computerspil', dep=Dependency.obj)],
                     [Token('Kasper', dep=Dependency.nsubj),
                      Token('loves', dep=Dependency.root),
                      Token('computerspil', dep=Dependency.obj)],
                     [Token('Martin', dep=Dependency.nsubj),
                      Token('enjoys', dep=Dependency.root),
                      Token('apples', dep=Dependency.obj)],
                     [Token('Lars', dep=Dependency.nsubj),
                      Token('hates', dep=Dependency.root),
                      Token('computerspil', dep=Dependency.obj)]]

        manual = Manual()

        kg_info = KnowledgeGraphInfo(sentences, manual)

        # Act
        self.kg.generate_triples(kg_info)
        result = self.kg.knowledge_graph_triples

        # Assert
        assert isinstance(result, list)
        assert len(result) == 4

    # 0 metadata
    # 1 title
    def test_generate_triple_for_publisher(self):
        # Arrange
        data = {
            "published_by": "some_publisher",
            "title": "manual123"
        }
        self.__setup_data_in_kg(data)

        result = self.kg.knowledge_graph_triples[0]

        expected = knox_triples.PublishTriple('manual123', 'some_publisher')

        # Assert
        assert result.publisher == expected.publisher
        assert result.manual_uri == expected.manual_uri

    def test_generate_triple_for_published_at(self):
        data = {
            "published_by": "some_publisher",
            "title": "manual123",
            "published_at": "2020-11-13"
        }

        self.__setup_data_in_kg(data)
        result = self.kg.knowledge_graph_triples[1]

        expected = knox_triples.PublishedAtTriple("manual123", "2020-11-13")

        # Assert
        assert result.published_at == expected.published_at
        assert result.manual_uri == expected.manual_uri

    def test_generate_triple_for_title(self):
        data = {
            "published_by": "grundfos",
            "title": "manual1337",
            "published_at": "2020-11-14"
        }

        self.__setup_data_in_kg(data)
        result = self.kg.knowledge_graph_triples[3]

        expected = knox_triples.TitleTriple("manual1337")

        # Assert
        assert result.title == expected.title
        assert result.manual_uri == expected.manual_uri

    def test_generate_triple_for_section(self):
        data = {
            "title": "manual123",
            "sections": [{
                "header": "some_header"
            }]
        }

        self.__setup_data_in_kg(data)

        assert self.kg.knowledge_graph_triples[2].section_title == "some_header"

    def test_generate_triples_for_multiple_sections(self):
        data = {
            "sections": [
                {
                    "header": "some_header"
                },
                {
                    "header": "some_header2"
                }
            ]
        }

        self.__setup_data_in_kg(data)
        result = self.kg.knowledge_graph_triples

        assert result[0].section_title == "some_header"
        assert result[1].section_title == "some_header2"

    def test_generate_triple_for_header_and_page(self):
        data = {
            "sections": [{
                "page": "some_page",
                "header": "some_header"
            }]
        }

        self.__setup_data_in_kg(data)

    def __setup_data_in_kg(self, data):
        new_data = {'content': data}
        manual = load_json(StringIO(json.dumps(new_data)))
        sentences = []
        self.kg.generate_triples(KnowledgeGraphInfo(sentences, manual))



from resources import knox_triples
from resources.json_wrapper import Section, Manual
from word_embedding.dependency import Dependency


def generate_triples_for_metadata(content: Manual):
    triples = []

    triples.extend(generate_triples_for_manual(content))
    triples.extend(generate_triples_for_sections(content))

    return triples


def generate_triples_for_manual(manual: Manual):
    triples = []

    if manual.published_by:
        triples.append(knox_triples.PublishTriple(manual.title, manual.published_by))

    if manual.published_at:
        triples.append(knox_triples.PublishedAtTriple(manual.title, manual.published_at))

    if manual.title:
        triples.append(knox_triples.MetaDataTriple(manual.title))
        triples.append(knox_triples.TitleTriple(manual.title))

    if manual.sections:
        for sec in manual.sections:
            triples.append(knox_triples.SectionTriple(manual.title, sec.header))

    return triples


def generate_triples_for_sections(manual: Manual):
    triples = []

    if manual.sections:
        for sec in manual.sections:

            if __has_header_and_page(sec):
                triples.append(knox_triples.PageTriple(manual.title, sec.page))
                triples.append(knox_triples.PageInSectionTriple(manual.title, sec.page, sec.header))

    return triples


def __has_header_and_page(section: Section):
    return section.header and section.page


def generate_triples_for_sentences(sentences):
    triples = []

    for sentence in sentences:
        triples.append(__process_sentence(sentence))

    return triples


def __process_sentence(sentence):
    subj = ''
    obj = ''
    relation = ''

    for token in sentence:
        if __is_subject(token) and __is_empty(subj):
            subj = token.name
        elif __is_object(token) and __is_empty(obj):
            obj = token.name
        elif __is_relation_candidate(token):
            relation = token.name
        else:
            continue

    return __parse_obj_relation_subj(subj, relation, obj)


def __parse_obj_relation_subj(subj, relation, obj):
    return knox_triples.SentenceTriple(subj, relation, obj)


def __is_subject(token):
    return token.dep == Dependency.nsubj or token.dep == Dependency.csubj or \
           token.dep == Dependency.csubjpass or token.dep == Dependency.nsubjpass


def __is_object(token):
    return token.dep == Dependency.obj or token.dep == Dependency.dobj or \
           token.dep == Dependency.pobj


def __is_empty(text_field):
    return not text_field


def __is_relation_candidate(token):
    return token.dep == Dependency.root



import random

def random_percentage_count():
    """
    This is gimmick percentages for fun loading experience!!!
    """
    var = "%"
    forloopnumber = random.randint(1,10)
    listofnumbers = random.sample(range(0, 99), forloopnumber)
    if 69 not in listofnumbers:
        listofnumbers.append(69)
    listofnumbers.sort()

    for number in listofnumbers:
        print(number, var, sep='')
    if listofnumbers[0] < 5:
        print("420%")
    else:
        print("100%")



from enum import Enum, auto

class GrammarCategories(Enum):
    """
    Grammatical categories for knox data
    """
    subj = auto()
    obj = auto()
    rel = auto()
    adj = auto()
    noun = auto()
    other = auto()



from rdflib import URIRef
from rdflib.namespace import ClosedNamespace

GRUNDFOS = ClosedNamespace(
    uri=URIRef("grundfos/", "http://www.knoxproject.aau.dk/"),
    terms=[
        "Manual", "Organisation", "Author", "publishes", "Date", "Publisher", "Page",
        "Section", "Unit", "the_pump", "isWrittenBy", "isPublishedBy", "isPublishedAt", "mentionedOn",
        "isInSection", "Title", "Link", "Name", "PublicationDate", "pageNumber",
        "sectionTitle"]
)



from rdf_parser.rdf_parser import RdfParser
from resources import knox_triples


class TestRdfParser:
    def setup_method(self):
        self.rdf_parser = RdfParser("file://testing.namespace.dk/")

    def test_should_be_able_to_add_triple(self):
        self.rdf_parser.add_rdf_triple(knox_triples.MetaDataTriple("Title of manual"))
        assert len(self.rdf_parser.rdf_graph) == 1



import pytest
from rdflib import term, BNode, Literal, XSD

from rdf_parser import rdf_helper
from rdf_parser.rdf_helper import RdfHelperException


class TestRdfHelper:
    def setup_method(self):
        self.namespace = "file://testing.namespace.dk/"

    def test_can_get_term(self):
        rdf_helper.get_term("rdf", "type")

    def test_can_generate_blank_node(self):
        blank_node = rdf_helper.generate_rdf_blank_node()
        assert isinstance(blank_node, BNode)

    def test_should_generate_correct_uriref(self):
        uri_ref = rdf_helper.generate_rdf_uri_ref(self.namespace, "name of unit", ["manual"])
        assert uri_ref == term.URIRef("file://testing.namespace.dk/manual/name_of_unit")

    def test_generate_correct_uriref_no_suburis(self):
        uri_ref = rdf_helper.generate_rdf_uri_ref(self.namespace, "some_title", [])
        assert uri_ref == term.URIRef("file://testing.namespace.dk/some_title")

    def test_generate_correct_uriref_empty_suburis(self):
        uri_ref = rdf_helper.generate_rdf_uri_ref(self.namespace, "some_title", [""])
        assert uri_ref == term.URIRef("file://testing.namespace.dk/some_title")

    def test_raise_exception_if_ref_is_empty(self):
        uri_ref = rdf_helper.generate_rdf_uri_ref(self.namespace, "", ["manual"])
        assert isinstance(uri_ref, BNode)

    def test_should_generate_correct_rdf_literal(self):
        literal = rdf_helper.generate_rdf_literal("Some String", "string")
        assert isinstance(literal, Literal)
        assert literal.datatype == XSD.string

        literal = rdf_helper.generate_rdf_literal("Some String", "date")

        assert isinstance(literal, Literal)
        assert literal.datatype == XSD.date

        literal = rdf_helper.generate_rdf_literal("Some String", "integer")

        assert isinstance(literal, Literal)
        assert literal.datatype == XSD.int

    def test_should_raise_rdf_helper_exception_on_empty_string(self):
        with pytest.raises(RdfHelperException):
            literal = rdf_helper.generate_rdf_literal("", "string")


from argparse import ArgumentParser
from mi_graph.util import setup_parser


class TestArgsParser:
    def setup_method(self):
        parser = ArgumentParser()
        self.mi_parser = setup_parser(parser)

    def test_parses_visual_option(self):
        args = ['--visualisation', 'file']

        parser = self.mi_parser.parse_args(args)

        assert parser.visualisation == True

    def test_visual_have_help(self):
        helper_msg = self.mi_parser.format_help()

        assert "visualizes the graph" in helper_msg

    def test_visual_default_is_false(self):
        empty_args = ['file']

        parser = self.mi_parser.parse_args(empty_args)

        assert parser.visualisation == False



import os

import pprint
import pandas as pd
import networkx as nx
import matplotlib.pyplot as plt
import requests

from mi_graph import triple_generation
from rdf_parser.rdf_parser import RdfParser
from resources.error import error
from resources.knowledgegraph_info_container import KnowledgeGraphInfo


class KnowledgeGraph:
    """
    Generates triples from the text which has been natural language processed,
    and the metadata extracted from a Content object.
    Triples are saved to the database(.csv) file provided to the object during instantiation.
    """
    knowledge_graph_triples = []

    def __init__(self):
        self.rdf_parser = RdfParser()
        self.knowledge_graph_triples = []

    def generate_triples(self, kg_info: KnowledgeGraphInfo):
        """
        Generates knox_triples (subj, rel, obj) from the text which has been natural language processed
        and the metadata extracted from a Content object
        """

        self.knowledge_graph_triples.extend(
            triple_generation.generate_triples_for_metadata(kg_info.manual))
        self.knowledge_graph_triples.extend(
            triple_generation.generate_triples_for_sentences(kg_info.sentences))

    def show_graph(self):
        """
        Opens a new window with the graph plotted within
        """
        graph = nx.from_pandas_edgelist(
            self.__create_branches_from_triples(), 'subject', 'object', edge_attr=True, create_using=nx.MultiDiGraph())

        plt.figure(figsize=(12, 12))

        pos = nx.spring_layout(graph)
        nx.draw(graph, pos, edge_color='black',
                node_color='skyblue', alpha=0.9, with_labels=True)
        nx.draw_networkx_edge_labels(graph, pos=pos)

        plt.show()

    def __create_branches_from_triples(self):

        data_frame = pd.DataFrame(columns=["subject", "relation", "object"])
        for triple in self.knowledge_graph_triples:
            try:
                data_frame = data_frame.append(
                    pd.DataFrame(
                        {'subject': [triple.subj_()], 'relation': [triple.rel_()], 'object': [triple.obj_()]}))
            except KeyError as error:
                print(error)

        return data_frame

    def pretty_print_graph(self):
        for triple in self.knowledge_graph_triples:
            pprint.pprint(triple.parse())

    def save_to_csv(self, csv_file_path):
        triple_dataframe = self.__create_branches_from_triples()

        if not os.path.exists(csv_file_path):
            triple_dataframe.to_csv(csv_file_path, mode='a', index=False)
        elif os.stat(csv_file_path).st_size == 0:
            triple_dataframe.to_csv(csv_file_path, mode='a', index=False)
        else:
            triple_dataframe.to_csv(csv_file_path, mode='a', index=False, header=None)

    def save_to_database(self):
        data = self.__generate_data()

        self.__update_triples(data)
        self.__commit_triples()

    def __update_triples(self, data):
        # Remember: ssh username@student.aau.dk@knox-node02.srv.aau.dk -L 8080:localhost:8080
        update_url = 'http://127.0.0.1:8080/update/'

        update_header: dict = {'content-type': 'application/json; charset=utf-16'}
        response = requests.post(update_url, data=data.encode('utf-16'), headers=update_header)
        self.__print_response(response)

    def __commit_triples(self):
        commit_url = 'http://127.0.0.1:8080/commit/'

        response = requests.post(commit_url)
        self.__print_response(response)

    def __generate_data(self):
        data = ""
        for triple in self.knowledge_graph_triples:
            try:
                data += repr(triple) + " .\n"
            except KeyError as err:
                error("The key is wrong", err)
        return data

    @staticmethod
    def __print_response(response):
        print("Status code: ", response.status_code)
        print(response.text)



import re
import spacy
from preprocess.cleaner import Cleaner


class CleanerImp(Cleaner):
    """
    First implementation of super class
    """

    def bigrams(self, sentence: str) -> str:
        nlp = spacy.load('en_core_web_sm')
        doc = nlp(sentence)

        for noun_phrase in list(doc.noun_chunks):
            if noun_phrase.string.endswith(' '):
                bigram = noun_phrase.string
                # Remove trailing whitespace in noun_phrase to avoid:
                # "ice cream " --> "ice_cream_"
                # Insted of "ice cream " --> "ice_cream "
                bigram = bigram.strip(' ')
                bigram = bigram.replace(' ', '_')
                bigram += ' '
            else:
                bigram = noun_phrase.string
                bigram = bigram.strip(' ')
                bigram = bigram.replace(' ', '_')

            sentence = sentence.replace(noun_phrase.string, bigram)

        return sentence

    def to_lower(self, words):
        return words.lower()

    def remove_special_characters(self, txt):
        cleaned_text = re.sub("[^a-zA-Z. ]", '', txt)
        return cleaned_text

    def numbers_to_text(self, text):
        result = ""

        for token in text.split():
            if token.isdigit():
                result += self._textify_token(token)
            else:
                result += token
            result += " "

        return result.strip()

    def _textify_token(self, token):
        result = ""
        for digit in token:
            result += self._textify_number(digit)
            result += " "

        return result.strip()

    def _textify_number(self, digit):
        numbers = {
            '0': 'zero',
            '1': 'one',
            '2': 'two',
            '3': 'three',
            '4': 'four',
            '5': 'five',
            '6': 'six',
            '7': 'seven',
            '8': 'eight',
            '9': 'nine'
        }
        return numbers[digit]

    def remove_duplicates(self, str_list):
        return list(set(str_list))

    def lemmatize(self, words: str):
        # Initialize spacy 'en' model, keeping only tagger component needed for lemmatization
        nlp = spacy.load('en_core_web_sm', disable=['parser', 'ner'])

        doc = nlp(words)

        return " ".join([token.lemma_ for token in doc])


    def insert_pump_name(self, data, pump_name):
        data = data.replace("the_pump", pump_name)
        return data


# Tasks:
#
# Should $ be converted to " dollar" ??



from abc import ABC

from rdflib.term import Node

from rdf_parser import rdf_helper, GRUNDFOS


class Triple(ABC):
    """
    Data-structure for the relation data (subject, object, relation), used with the knowledge graph
    """

    def subj_(self) -> Node:
        pass

    def rel_(self) -> Node:
        pass

    def obj_(self) -> Node:
        pass

    def parse(self):
        return self.subj_(), self.rel_(), self.obj_()

    def term(self, namespace, term):
        return rdf_helper.get_term(namespace, term)

    def literal(self, string: str, literal_type: str):
        return rdf_helper.generate_rdf_literal(string, literal_type)

    def blank_node(self, string: str):
        return rdf_helper.generate_rdf_blank_node(string)

    def __repr__(self):
        return "<" + str(self.subj_()) + "> " + "<" + str(self.rel_()) + "> " + "<" + str(self.obj_()) + ">"


class StringTriple(Triple):
    """
    A triple where everything is strings
    """

    def __init__(self, _subj: str, _rel: str, _obj: str):
        self.subj = _subj
        self.obj = _obj
        self.rel = _rel


class MetaDataTriple(Triple):
    """
    The default triple for metadata
    """

    def __init__(self, title):
        self.manual_uri = rdf_helper.generate_rdf_uri_ref(GRUNDFOS.uri,
                                                          ref=title,
                                                          sub_uris=["manual"])

    def subj_(self):
        return self.manual_uri

    def rel_(self):
        return self.term("rdf", "type")

    def obj_(self):
        return self.term("grundfos", "Manual")

    def __repr__(self):
        return "<" + str(self.subj_()) + "> " + "<" + str(self.rel_()) + "> " + "<" + str(self.obj_()) + ">"


class PublishTriple(MetaDataTriple):
    """
    Define about the publisher of an article
    """

    def __init__(self, title: str, publisher: str = ''):
        super().__init__(title)
        self.publisher = publisher

    def subj_(self):
        return self.manual_uri

    def rel_(self):
        return self.term("grundfos", "isPublishedBy")

    def obj_(self):
        return self.literal(self.publisher, "string")

    def __repr__(self):
        return "<" + str(self.subj_()) + "> " + "<" + str(self.rel_()) + "> " + "\"" + str(self.obj_()) + "\""


class PublishedAtTriple(PublishTriple):
    """
    Define when a article is published
    """

    def __init__(self, title: str, published_at: str):
        super().__init__(title)
        self.published_at = published_at

    def rel_(self):
        return self.term("grundfos", "isPublishedAt")

    def obj_(self):
        return self.literal(self.published_at, "date")


class TitleTriple(MetaDataTriple):
    """
    Define a title of a manual
    """

    def __init__(self, title):
        super().__init__(title)
        self.title = title

    def rel_(self):
        return self.term("grundfos", "Title")

    def obj_(self):
        return self.literal(self.title, "date")

    def __repr__(self):
        return "<" + str(self.subj_()) + "> " + "<" + str(self.rel_()) + "> " + "\"" + str(self.obj_()) + "\""


class SectionTriple(Triple):
    """
    Define a section
    """

    def __init__(self, content_title, section_title):
        self.section_uri = rdf_helper.generate_rdf_uri_ref(GRUNDFOS.uri, ref=section_title,
                                                           sub_uris=["manual", content_title, "section"])

        self.section_title = section_title

    def subj_(self):
        return self.section_uri

    def rel_(self):
        return self.term("rdf", "type")

    def obj_(self):
        return self.term("grundfos", "Section")


class PageTriple(Triple):
    """
    Define a page
    """

    def __init__(self, content_title, page_number):
        self.page_uri = rdf_helper.generate_rdf_uri_ref(GRUNDFOS.uri,
                                                        ref=page_number,
                                                        sub_uris=["manual", content_title, "section", "page"])
        self.page_number = page_number

    def subj_(self):
        return self.page_uri

    def rel_(self):
        return self.term("rdf", "type")

    def obj_(self):
        return self.term("grundfos", "Page")


class PageInSectionTriple(Triple):
    """
    Defines a relation between a page and a section
    """

    def __init__(self, content_title, page_number, section_title):
        self.page_uri = rdf_helper.generate_rdf_uri_ref(GRUNDFOS.uri,
                                                        ref=page_number,
                                                        sub_uris=["manual", content_title, "section", "page"])
        self.section_uri = rdf_helper.generate_rdf_uri_ref(GRUNDFOS.uri,
                                                           ref=section_title,
                                                           sub_uris=["manual", content_title, "section"])

    def subj_(self):
        return self.page_uri

    def rel_(self):
        return self.term("grundfos", "isInSection")

    def obj_(self):
        return self.section_uri


class SentenceTriple(Triple):
    """
    Defines triples that extract knowledge from the triple
    """

    def __init__(self, subj, rel, obj):
        super().__init__()
        self.subj = subj
        self.rel = rel
        self.obj = obj

    def subj_(self):
        return self.term("grundfos", self.subj)

    def rel_(self):
        return self.term("grundfos", self.rel)

    def obj_(self):
        try:
            term = self.term("grundfos", self.obj)
        except KeyError:
            term = self.blank_node(self.obj)
        return term

    def __repr__(self):
        return "<" + str(self.subj_()) + "> " + "<" + str(self.rel_()) + "> " + "<" + str(self.obj_()) + ">"



import json

import requests

from resources.term_frequency import TermFrequency
from resources.json_wrapper import Manual
from resources.knowledgegraph_info_container import KnowledgeGraphInfo
from resources.random_number_gen import random_percentage_count

from preprocess.cleaner_imp import CleanerImp

from word_embedding.spacy_model import SpacyModel
from .knowledge_graph import KnowledgeGraph


def create_knowledge_graph(manual):
    corpus = extract_all_text_from_paragraphs(manual)
    corpus = clean_text(corpus)
    tokens = extract_tokens_from(corpus)
    kg_info = KnowledgeGraphInfo(tokens, manual)
    knowledge_graph = KnowledgeGraph()
    knowledge_graph.generate_triples(kg_info)
    knowledge_graph.save_to_database()
    return knowledge_graph


def clean_text(corpus):
    cleaner = CleanerImp()
    corpus = cleaner.remove_special_characters(corpus)
    corpus = cleaner.numbers_to_text(corpus)
    corpus = cleaner.lemmatize(corpus)
    corpus = cleaner.bigrams(corpus)
    corpus = cleaner.to_lower(corpus)
    return corpus


def extract_tokens_from(corpus):
    random_percentage_count()
    model = SpacyModel()
    model.load(corpus)
    tokens = model.tokens()
    return tokens


def extract_all_text_from_paragraphs(data: Manual):
    text: str = ''
    for sec in data.sections:
        text += sec.paragraph.text
    return text


def generate_word_counts(manual: Manual):
    cleaner = CleanerImp()
    word_counter = TermFrequency()
    for section in manual.sections:
        cleaned_text = cleaner.remove_special_characters(section.paragraph.text)
        cleaned_text = cleaner.to_lower(cleaned_text)
        word_counter.process(manual.title, cleaned_text)

    for doc in word_counter:
        form = {}
        words = [{'word': word, 'amount': frequency} for word, frequency in word_counter[doc].items()]
        form['words'] = words

        form['articletitle'] = doc

        form["filepath"] = f"/test/{doc}"

        form['totalwordsinarticle'] = word_counter[doc].length

        # ´har portforward port 8080, fra server til local host
        # ssh username@student.aau.dk@knox-node02.srv.aau.dk -L 8080:localhost:8080
        # url = 'http://knox-node02.srv.aau.dk/wordCountData/'
        url = 'http://127.0.0.1:8080/wordCountData/'
        json_form = json.dumps(form)
        header: dict = {'content-type': 'application/json; charset=utf-8'}
        response = requests.post(url, data=json_form, headers=header)
        print("Status code: ", response.status_code)
        print("Printing Entire Post Request")
        print(response)


def run(manual, visualize=False):
    knowledge_graph = create_knowledge_graph(manual)
    generate_word_counts(manual)

    # If --visualisation" or "-v" in args
    if visualize:
        print("Rendering knowledge graph in new window...")
        knowledge_graph.show_graph()
        print("...Visualization closed\n\n")

    return knowledge_graph



import pytest
from word_embedding.grammarcategories import GrammarCategories
from word_embedding.spacy_model import SpacyModel
from word_embedding.token import Token
from word_embedding.dependency import Dependency


class TestModel:

    def setup_method(self):
        self.model = SpacyModel()

    def test_load_test_into_model_produce_keyvectors(self):
        cleaned_sentence = "original safety instruction"

        self.model.load(cleaned_sentence)

        assert self.model.keyvector is not None

    def test_spacy_model_exists(self):
        assert self.model is not None

    def test_input_sentence_in_tokens(self):
        cleaned_sentence = "original safety instruction"
        self.model.load(cleaned_sentence)

        token = Token("original")
        tokens = self.model.tokens()

        assert token in tokens[0]

    def test_token_has_label(self):
        self.model.load("apple")

        first_sentance= self.model.tokens()[0]
        token = first_sentance.pop()

        assert token.pos_tag == GrammarCategories.noun

    def test_token_can_have_adj_label(self):
        self.model.load("good")

        first_sentance = self.model.tokens()[0]
        token =  first_sentance.pop()

        assert token.pos_tag == GrammarCategories.adj

    def test_token_out_of_category(self):
        self.model.load("tyggegummi")

        tokens = self.model.tokens()
        token = tokens[0].pop()

        assert token.pos_tag == GrammarCategories.other

    def test_detect_dependency_nsubj(self):
        self.model.load("Apple is looking at buying U.K. startup for $1 billion")

        tokens = self.model.tokens()
        apple = tokens[0][0]

        assert apple.dep == Dependency.nsubj

    def test_detect_dependency_aux(self):
        self.model.load("Apple is looking at buying U.K. startup for $1 billion")

        tokens = self.model.tokens()
        _is = tokens[0][1]

        assert _is.dep == Dependency.aux

    def test_detect_dependency_root(self):
        self.model.load("Apple is looking at buying U.K. startup for $1 billion")

        first_sentance= self.model.tokens()[0]
        looking = first_sentance[2]

        assert looking.dep == Dependency.root

    def test_detect_dependency_prep(self):
        self.model.load("Apple is looking at buying U.K. startup for $1 billion")

        first_sentance= self.model.tokens()[0]
        at = first_sentance[3]

        assert at.dep == Dependency.prep

    def test_detect_dependency_pcomp(self):
        self.model.load("Apple is looking at buying U.K. startup for $1 billion")

        first_sentance= self.model.tokens()[0]
        buying = first_sentance[4]

        assert buying.dep == Dependency.pcomp

    def test_detect_dependency_compund(self):
        self.model.load("Apple is looking at buying U.K. startup for $1 billion")

        first_sentance= self.model.tokens()[0]
        UK = first_sentance[5]

        assert UK.dep == Dependency.compound

    def test_detect_dependency_dobj(self):
        self.model.load("Apple is looking at buying U.K. startup for $1 billion")

        first_sentance= self.model.tokens()[0]
        startup = first_sentance[6]

        assert startup.dep == Dependency.dobj

    def test_detect_dependency_quantmod(self):
        self.model.load("Apple is looking at buying U.K. startup for $1 billion")

        tokens = self.model.tokens()
        dollar = tokens[0][8]

        assert dollar.dep == Dependency.quantmod

    def test_detect_dependency_pobj(self):
        self.model.load("Apple is looking at buying U.K. startup for $1 billion")

        tokens = self.model.tokens()
        billion = tokens[0][10]

        assert billion.dep == Dependency.pobj

    def test_can_split_text_into_one_list_per_sentence(self):
        self.model.load("I like apples. He likes apples. She likes Apples.")

        tokens = self.model.tokens()

        i = tokens[0][0]
        assert i.name == "I"

        he = tokens[1][0]
        assert he.name == "He"

        she = tokens[2][0]
        assert she.name == "She"

    def test_can_split_text_into_one_list_per_sentence_no_punctuation(self):
        self.model.load("i like apples. he likes apples. she likes apples")

        tokens = self.model.tokens()

        i = tokens[0][0]
        assert i.name == "i"

        he = tokens[1][0]
        assert he.name == "he"

        she = tokens[2][0]
        assert she.name == "she"



import pytest
from io import StringIO
from loader.file_loader import load_json
import json

from resources.json_wrapper import recursive_parse_section


class TestLoader:

    def setup_method(self):
        self.json_file_path = ""

    def test_json_load_returns_dict(self):
        # Arrange
        fake_file = '{}'
        with StringIO(fake_file) as json_file:
            result = json.load(json_file)

        # Assert
        assert isinstance(result, dict)

    def test_load_json_file_throws_exception_if_json_file_is_empty(self):
        # Arrange
        empty_file_path = ''

        # Act

        # Assert
        with pytest.raises(IOError):
            load_json(empty_file_path)

    def test_assign_correct_publisher_to_content(self):
        fake_file = StringIO('{"content": {"published_by": "some_publisher"}}')
        expected = "some_publisher"

        result = load_json(fake_file)

        assert result.published_by == expected

    def test_assign_correct_published_at_to_content(self):
        # Arrange
        fake_file = StringIO('{"content": {"published_at": "2020-23-07"}}')
        expected = '2020-23-07'

        # Act
        result = load_json(fake_file)

        # Assert
        assert result.published_at == expected

    def test_assign_correct_title_to_content(self):
        fake_file = StringIO('{"content": {"title": "ALPHA1"}}')
        expected = "ALPHA1"

        result = load_json(fake_file)

        assert result.title == expected

    def test_can_extract_sections_to_content(self):
        # Arrange
        fake_file = StringIO('{"content": { "sections": [{ }] }}')

        # Act
        result = load_json(fake_file)

        # Assert
        assert result.sections is not None

    def test_assign_correct_page_to_section(self):
        fake_file = StringIO('{"content":{ "sections": [{"page": "2", "header": "info"}] }}')
        expected_page = "2"

        result = load_json(fake_file)
        first_section = 0
        assert (result.sections[first_section].page == expected_page)

    def test_assign_correct_header_to_section(self):
        # Arrange
        fake_file = StringIO('{"content":{ "sections": [{"header": "hello"}] }}')
        expected = "hello"
        # Act
        result = load_json(fake_file)

        # Assert
        assert (result.sections[0].header == expected)

    def test_assign_correct_page_to_paragraph(self):
        # Arrange
        fake_file = StringIO('{"content": { "sections": [{ "paragraphs": {"page": "3"} }] }}')
        expected = "3"
        # Act
        result = load_json(fake_file)
        # Assert
        assert (result.sections[0].paragraph.page == expected)

    def test_assign_correct_text_to_paragraph(self):
        # Arrange
        fake_file = StringIO('{"content": {"sections": [{"paragraphs": {"text": "hello world"}}]}}')
        expected = "hello world"
        # Act
        result = load_json(fake_file)

        # Assert
        assert (result.sections[0].paragraph.text == expected)

    def test_parse_two_recursive_sections(self):
        content = StringIO('{"content": { "sections": [{"sections": [{"info": "test_2"}],"info": "test_1"}]}}')

        full_manual = load_json(content)

        assert len(full_manual.sections) == 2


def make_grundfos_schema(mid: str):
    start = '{"properties": {"properties": {'
    end = '}}}}'

    return StringIO(start + mid + end)


class TestRecursiveParser:
    def test_recurse_section_base_case(self):
        data = {"sections": [{"info": "test"}]}

        list_of_sections = recursive_parse_section(data)

        assert len(list_of_sections) == 1
        assert list_of_sections[0]['info'] == "test"

    def test_recurse_section_two_sections(self):
        data = {
            "sections": [{
                "sections": [{
                    "info": "test_2"}],
                "info": "test_1"}]
        }

        list_of_sections = recursive_parse_section(data)

        assert list_of_sections[0]['info'] == "test_1"
        assert list_of_sections[1]['info'] == "test_2"

    def test_recurse_section_three_sections(self):
        data = {
            "sections": [{
                "sections": [{
                    "sections": [{
                        "info": "test_3"}],
                    "info": "test_2"}],
                "info": "test_1"}]
        }

        list_of_sections = recursive_parse_section(data)

        assert list_of_sections[0]['info'] == "test_1"
        assert list_of_sections[1]['info'] == "test_2"
        assert list_of_sections[2]['info'] == "test_3"

    def test_first_section_do_not_have_section(self):
        data = {
            "sections": [{
                "sections": [{
                    "info": "test_2"}],
                "info": "test_1"}]
        }

        list_of_sections = recursive_parse_section(data)
        print(list_of_sections)
        assert "sections" not in list_of_sections[0].keys()



from typing import List
from resources.json_wrapper import Manual


class KnowledgeGraphInfo:
    """
    Data-structure for the information needed to construct a knowledge graph object
    """
    sentences: List[list]
    manual: Manual

    def __init__(self, _sentences=None, _manual=None):
        self.sentences = _sentences
        self.manual = _manual



from rdflib import Graph, RDF
from rdflib.namespace import RDFS, OWL

from rdf_parser import GRUNDFOS
from resources.error import error


class RdfParser:
    """
    Used for parsing custom namespace and ontology,
    this allows using relationships that works for the custom contexts.
    For the KNOX project it would either be Grundfos or Nordjyske.

    Methods
    -------
    use_ontology_from_path(file_path, file_ext)
        This method loads a given ontology and parses it to a specified format.
        If no file_ext is specified, the default is "ttl" (turtle)

    Attributes
    ----------
    self.GRUNDFOS : rdflib.ClosedNamespace
        Namespace for the context of Grundfos, it contains the relevant relations as well as types
    self.rdf_graph : rdflib.Graph
        Holds the entire graph, in this case it is the knowledge graph for Grundfos.
    """
    rdf_graph = Graph()

    def __init__(self, namespace_base="http://www.knoxproject.aau.dk/"):
        self.rdf_graph = Graph()
        self.rdf_graph.bind("grundfos", GRUNDFOS)
        self.rdf_graph.bind("rdf", RDF)
        self.rdf_graph.bind("rdfs", RDFS)
        self.rdf_graph.bind("owl", OWL)
        self.namespace_base = namespace_base

    def add_rdf_triple(self, rdf_triple):
        try:
            triple = rdf_triple.parse()
            self.rdf_graph.add(triple)
        except KeyError as err:
            error("the key is wrong", err)

    def store_knowledge_graph(self, output_folder_path, output_file_name, output_format):
        """Stores the knowledge graph in a file in a user specified location.

        Parameters
        ----------
        output_file_name : str
            Specify the output file name
        output_folder_path : str
            Specify the wanted output folder - this stores the knowledge graph locally.
        output_format : str
            Specify the output format for the knowledge graph to be saved as.
        """
        self.rdf_graph.serialize(format=output_format,
                                 destination=output_folder_path + output_file_name + "." + output_format,
                                 encoding="utf-8")

    def pretty_print_knowledge_graph(self):
        """This method is used to print out the knowledge graph in a human readable way.

        Be aware that it goes through every triple in the knowledge graph
        """
        print(self.rdf_graph.serialize(format="turtle").decode("utf-8"))



from typing import List


class Manual:
    """
    The data structure for the manual.
    This is the only one that should be used in the rest of the project

    """

    def __init__(self, title='', publisher='', published_at='', sections=''):
        self.title: str = title
        self.published_by: str = publisher
        self.published_at: str = published_at
        self.sections: [] = sections


class Paragraph:
    """
    Data-structure for the information stored under
    each paragraph in the "paragraph" array in the json file
    """
    page: str = ''
    text: str = ''

    def __init__(self, data: dict = None):
        if data:
            self.page = data.get("page", "")
            self.text = data.get("text", "")


class Section:
    """
    Data-structure for the information stored under
    each section in the "sections" array in the json file
    """

    def __init__(self, data: dict = None):
        if data:
            self.page = data.get("page", "")
            self.header = data.get("header", "")

            if 'paragraphs' in data.keys():
                json_paragraph = data.get("paragraphs", "")
                self.paragraph = Paragraph(json_paragraph)


class Content:
    """
    Data-structure for the information stored under "content" in the json file
    """
    published_by: str = ''
    published_at: str = ''
    title: str = ''
    sections: List[Section] = []

    def __init__(self, data: dict = None):

        if data:
            self.published_by = data.get("published_by")

            if publish_date := data.get("published_at"):
                self.published_at = publish_date

            self.title = data.get("title")

            self.sections: List[Section] = []

            if 'sections' in data.keys():
                json_sections = recursive_parse_section(data)
                for section in json_sections:
                    self.sections.append(Section(section))


def recursive_parse_section(data: {}) -> []:
    if "sections" in data.keys():
        array_of_sections = data.get('sections')
        for section in array_of_sections:
            new_flat_array_of_section = recursive_parse_section(section)

            if "sections" in section.keys():
                del section['sections']

            array_of_sections = array_of_sections + new_flat_array_of_section

        return array_of_sections

    return []



import spacy
from word_embedding.grammarcategories import GrammarCategories
from word_embedding.converter import Converter
from word_embedding.token import Token


class SpacyModel:
    """
    This is a implementations of the abstract class model, using spacy
    """
    def load(self, sentence: str):
        self.words = sentence
        self.spacytokens = self.model(sentence)

    def __init__(self):
        self.model = spacy.load("en_core_web_sm")
        self.spacytokens = None

    words = ""
    keyvector = ""

    def tokens(self):
        doc = list(self.spacytokens.sents)
        sentences = []

        for sentence in doc:
            sentences.append(self.__convert_tokens(sentence))

        return sentences

    def __convert_tokens(self, sentence):
        list_of_tokens = []

        for spacy_token in sentence:
            name = spacy_token.text
            pos_tag = self.get_pos_tag_from_spacytoken(spacy_token)
            dependency = self.get_dependency_from_spacytoken(spacy_token)

            new_token = Token(name, pos_tag, dependency)
            list_of_tokens.append(new_token)

        return list_of_tokens

    def get_dependency_from_spacytoken(self, spacy_token):
        spacy_dependency = spacy_token.dep_
        return Converter.dependency(spacy_dependency)

    def get_pos_tag_from_spacytoken(self, spacy_token):
        space_pos_tag = spacy_token.tag_
        converter = {
            "NN": GrammarCategories.noun,
            "JJ": GrammarCategories.adj,
            "NNP": GrammarCategories.noun}
        if space_pos_tag not in converter.keys():
            return GrammarCategories.other

        return converter[space_pos_tag]



import setuptools
from mi_graph import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()


def requirements():
    return [i.strip() for i in open("requirements.txt").readlines()]


setuptools.setup(
    name="knox-mi-graph",  # Replace with your own username
    version=__version__,
    author="Foersteholdet",
    author_email="sw514e20@cs.aau.dk",
    description="Knowledge graph builder",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://git.its.aau.dk/Knox/mi-graph",
    packages=setuptools.find_packages(),
    install_requires=requirements(),
    entry_points={"console_scripts": ['mi_graph=mi_graph:cli']},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
)
# AAU schema
# https://repos.libdom.net/



from collections import Counter


class TermFrequency:
    """
    A module that count the term frequency.

    It is used by processing strings, where a doc title is given, and the string is given
    The text is expected to be preprocessed

    ex.
    from resources.term_frequency import TermFrequency
    tf = TermFrequency()
    tf.process(doc.title, doc.corpus)

    Then it can accessed like a hashtable

    tf.terms[word] gives an array of the documents it is found in.
    tf[doc_title] gives a hashtable where the key is term, and the value is frequency
    """
    def __init__(self):
        self.terms = {}
        self.__doc = {}

    def __getitem__(self, key):
        if key not in self.__doc.keys():
            self.__doc[key] = DocCounter()
        return self.__doc[key]

    def __iter__(self):
        return iter(self.__doc)

    def process(self, doc_title: str, sentence: str):
        """
        Process text, given a document title
        """
        for word in sentence.split(' '):
            self[doc_title].length += 1
            self[doc_title][word] += 1

            if word not in self.terms.keys():
                self.terms[word] = []
            if doc_title not in self.terms[word]:
                self.terms[word].append(doc_title)


class DocCounter(Counter):  # pylint: disable=W0223
    # Abstract method is not overwritten. The Counter object, choose, not to implement it.
    """
    A counter object with a added length and vocab var
    """
    def __init__(self, *args, **kwargs):
        self.length = 0
        super().__init__(*args, **kwargs)

    @property
    def vocab(self):
        return len(self.keys())



import json
from io import StringIO

from flask import Flask, Response, request

from loader.file_loader import load_json
from mi_graph import pipeline

app = Flask(__name__)


def start_api():
    app.run(debug=True)


@app.route('/api/manuals', methods=['POST'])
def process_manual():
    manual = create_manual_from_request(request.json)
    try:
        pipeline.run(manual)
    except Exception as error:  # pylint: disable=W0703
        return Response(response=str(error),
                        status=400,
                        content_type='application/json')

    return Response(response=json.dumps({"success": True}),
                    status=201,
                    content_type='application/json')


def create_manual_from_request(json_data):
    json_dump = json.dumps(json_data)
    data_as_io_stream = StringIO(json_dump)
    manual = load_json(data_as_io_stream)
    return manual



from rdflib import RDF, RDFS, OWL, XSD, BNode, URIRef, Literal

from rdf_parser import GRUNDFOS
from resources.error import error


def get_term(namespace, term):
    """ Finds and return a closed namespace term when given a string and a namespace

    Make sure the term is within the namespaces.
    Parameters
    ----------
    namespace : str
        Namespace containing the wanted term
    term : str
        Term to find in given namespace

    Raises
    ------
    Exception
        Could not find namespace

    Returns
    -------
    closed_namespace_term
        A closed namespace term
    """
    if namespace == "rdf":
        return RDF.term(term)
    if namespace == "rdfs":
        return RDFS.term(term)
    if namespace == "owl":
        return OWL.term(term)
    if namespace == "xsd":
        return XSD.term(term)
    if namespace == "grundfos":
        return GRUNDFOS.term(term)

    raise Exception("Namespace not found")


def generate_rdf_blank_node(value=None):
    """ Returns a RDF BlankNode (BNode)

    Returns
    -------
    BNode
        A RDF BlankNode. Can only be used to as Object and Subject in an RDF Triple.
    """
    return BNode(value)


def generate_rdf_uri_ref(namespace_base, ref, sub_uris: list):
    """ Returns a new URI reference

    Parameters
    ----------
    ref : str
        Reference to RDF Object
    sub_uris : array of str
        Sub URI's of the reference e.g. ["manual", "section"]
    namespace_base : The base of the namespace used to generate the UriRef
        An example is "https://knox-project.aau.dk"

    Returns
    -------
    URIRef
        A URIRef to the RDF Object

    Example
    -------
    generate_rdf_uri_ref("section xzy", ["manual", "section"])
    # Returns the URIRef -> "file://testnamepace.org/manual/section/section_xzy"
    """
    if not ref:
        return generate_rdf_blank_node()

    uri_builder = ""

    for uri in sub_uris:
        if not uri:
            continue
        uri_builder += uri + "/"

    uri_builder += ref
    uri_builder = uri_builder.replace(" ", "_")

    return URIRef(namespace_base + uri_builder)


def generate_rdf_literal(value: str, literal_type: str) -> Literal:
    if value in "" or value is None:
        error("Empty value in rdf literal is not supported")
        raise RdfHelperException()

    if literal_type in "string":
        return Literal(value.replace(" ", "_"), datatype=XSD.string)

    if literal_type in "date":
        return Literal(value, datatype=XSD.date)

    if literal_type in "integer":
        return Literal(value, datatype=XSD.int)

    raise TypeError()


class RdfHelperException(Exception):
    """
    wrapper to rdf exception, so we don't depend on that library
    """



import abc
from abc import ABC


class Cleaner(ABC):
    """
    A class that have the responsibility to clean strings.
    It handles preprocessing
    """
    @abc.abstractmethod
    def bigrams(self, sentence: str) -> str:
        """
        returns list of bigram, from a string
        a bigram is a concatenation of a noun_chunk e.g: "New York" --> "New_York"



        :param sentence: str
        :return: sentence: str
                """

    @abc.abstractmethod
    def to_lower(self, words) -> str:
        """
        makes a string to lowercase characters

        :param words:
        :return: the input string to lower
        """

    @abc.abstractmethod
    def remove_special_characters(self, txt: str) -> str:
        """
        Removes special characters from a string

        :param txt:
        :return:
        """

    @abc.abstractmethod
    def lemmatize(self, words: str) -> str:
        """
        lemmatizes all words in a string full of words

        :param words:
        :return: words lemmatized, same format as string
        """

    def remove_duplicates(self, str_list: [str]) -> [str]:
        """


        :param str_list: a list of strings, where you want duplicates removed
        :return: a list of strings, with the duplicates removed
        """



from resources.term_frequency import TermFrequency


class TestTermFrequency:
    def setup_method(self):
        self.term_index = TermFrequency()
        self.doc = 'doc1'

    def test_count_a_word_which_is_processed(self):
        term = 'word'
        self.term_index.process(self.doc, term)

        frequency = self.term_index[self.doc][term]

        assert frequency == 1

    def test_second_word_in_sentence_is_processed(self):
        sentence = 'the house'
        second_word = 'house'
        self.term_index.process(self.doc, sentence)

        frequency = self.term_index[self.doc][second_word]

        assert frequency == 1

    def test_count_two_of_the_same_words(self):
        sentence = 'a very very big house'
        word = 'very'
        self.term_index.process(self.doc, sentence)

        frequency = self.term_index[self.doc][word]

        assert frequency == 2

    def test_process_two_sentences_from_different_docs(self):
        word = 'sentence'
        doc_1 = 'doc1'
        doc_2 = 'doc2'

        self.term_index.process(doc_1, word)
        self.term_index.process(doc_2, word)
        doc_from_words = self.term_index.terms[word]

        assert all(doc in doc_from_words
                   for doc in [doc_1, doc_2])

    def test_get_documents_where_a_word_is_found(self):
        word = 'word'
        self.term_index.process(self.doc, word)

        doc_from_words = self.term_index.terms[word]

        assert self.doc in doc_from_words

    def test_count_all_words_in_doc(self):
        five_words = "lorem ipsum dolor sit amet"
        doc = 'doc1'
        self.term_index.process(doc, five_words)

        length = self.term_index[doc].length

        assert length == len(five_words.split(' '))

    def test_count_vocab_in_document(self):
        two_different_words = "lorem lorem ipsum"
        doc = 'doc1'
        self.term_index.process(doc, two_different_words)

        vocab = self.term_index[doc].vocab

        assert vocab == 2

    def test_title_appear_in_corpus(self):
        corpus = "D42 is a pump"
        doc = "D42"
        self.term_index.process(doc, corpus)

        word_document = self.term_index.terms["D42"]

        assert doc in word_document

    def test_doc_do_not_appear_twice_lookup_word(self):
        corpus = "the the"
        doc = "D42"
        self.term_index.process(doc, corpus)

        word_documents = self.term_index.terms["the"]

        assert len(word_documents) == 1

